## Copyright  1999 CodeIt Computing, All rights reserved.

## By obtaining, using, and/or copying this software and/or its
## associated documentation, you agree that you have read, understood,
## and will comply with the following terms and conditions:

## Permission to use, copy, modify, and distribute this software and its
## associated documentation for any purpose and without fee is hereby
## granted, provided that the above copyright notice appears in all
## copies, and that both that copyright notice and this permission notice
## appear in supporting documentation, and that the name of CodeIt
## Computing not be used in advertising or publicity pertaining to
## distribution of the software without specific, written prior
## permission.

## CODEIT COMPUTING DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS
## SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND
## FITNESS. IN NO EVENT SHALL CODEIT COMPUTING BE LIABLE FOR ANY SPECIAL,
## INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING
## FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
## NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
## WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

__version__='$Revision: 1.2 $'[11:-2]

from ZCCcrypt.ZCCcrypt import encrypt, decrypt
from WampumExceptions import *

from base64 import encodestring, decodestring
from urllib import quote_plus, unquote_plus, urlopen
from types import DictType
from time import strptime, strftime, mktime, timezone, altzone, daylight, localtime
import string


class Register:

    def _SendServer(self, operation, msg):
	message = encodedict(msg)
	skey, mac, message = encrypt(self.merchant_key, message)
	url = "%scr21api.cgi/%s/%s" % (self.ccps_host, operation, self.cybercash_id)
	
	eMessage = {} # The encrypted message
	eMessage['session-key'] = skey
	eMessage['mac'] = encodestring(mac)
	eMessage['message'] = encodestring(message)

	conn = urlopen(url, encodedict(eMessage))
	response = decodedict(string.strip(conn.readline()))
	response =  decrypt(self.merchant_key, 
			    response['session-key'], 
			    decodestring(response['mac']), 
			    decodestring(response['message']))
	
	response = decodedict(response)
	status = response['MStatus']
	if  status == 'success':
	    return response
	elif status == 'success-duplicate':
	    raise SuccessDuplicate(response)
	elif status == 'partial-success':
	    raise PartialSuccess(response)
	elif status == 'failure-hard':
	    raise FailureHard(response)
	elif status == 'failure-q-or-cancel' or status == 'failure-q-or-discard':
	    raise FailureQ(response)
	elif status == 'failure-swversion':
	    raise FailureSWVersion(response)
	elif status == 'failure-bad-money':
	    raise FailureBadMoney(response)
	else:				# Unknown status
	    raise WampumException(response)

    def batchprep(self, *objects, **named_attrs):
	targets = (named_attrs,) + objects
	msg = {}

	addCCField(msg, 'order-id',    targets)
	addCCField(msg, 'txn-type',    targets)
	addCCField(msg, 'card-type',    targets)
	addCCField(msg, 'start-time',    targets)
	addCCField(msg, 'end-time',    targets)
	addCCField(msg, 'low-amount',    targets)
	addCCField(msg, 'high-amount',    targets)
	addCCField(msg, 'maxcount',    targets)

	# Remove fields with no values
	for key, val in msg.items():
	    if not string.strip(val):
		del msg[key]
		
	result =  self._SendServer('batch-prep', msg)

	results = []
	for key, val in result.items():
	    if key[0] == 'a' and (47 < ord(key[1]) < 58):
		results.append(CyberCashResponse(decodedict(val)))
		
	return results

    def batchcommit(self, ids, *objects, **named_attrs):
	targets = (named_attrs,) + objects
	msg = {}
	
	msg['num-txns'] = str(len(ids))
	for id in ids:
	    addCCField(msg, 'order-id-%s' % id, targets, 1)
	    addCCField(msg, 'txn-type-%s' % id, targets, 1)
	    addCCField(msg, 'amount-%s'   % id, targets, 1)


	result =  self._SendServer('batch-commit', msg)

	return CyberCashResponse(result)

		     

    def processorder(self, *objects, **named_attrs):
	targets = (named_attrs,) + objects
	msg = {}

	addCCField(msg, 'order-id',    targets, 1)
	addCCField(msg, 'amount',      targets, 1)
	addCCField(msg, 'card-number', targets, 1)
	addCCField(msg, 'card-exp',    targets, 1)

	addCCField(msg, 'card-name',   targets)
	addCCField(msg, 'card-address', targets)
	addCCField(msg, 'card-city', targets)
	addCCField(msg, 'card-zip', targets)
	addCCField(msg, 'card-state', targets)
	addCCField(msg, 'card-country', targets)
	
	result =  self._SendServer(self.txn_model, msg)

	if self.require_avs and (result['avs-code'] != 'Y'):
	    raise AVSFailed(result)
	
	return CyberCashResponse(result)

    def postauth(self, *objects, **named_attrs):
	targets = (named_attrs,) + objects
	msg = {}

	addCCField(msg, 'order-id',    targets, 1)
	addCCField(msg, 'amount',      targets, 1)
	
	result =  self._SendServer('postauth', msg)

	return CyberCashResponse(result)

    def void(self, *objects, **named_attrs):
	targets = (named_attrs,) + objects
	msg = {}

	addCCField(msg, 'order-id',    targets, 1)
	addCCField(msg, 'txn-type',      targets, 1)
	
	result =  self._SendServer('void', msg)

	return CyberCashResponse(result)


    def query(self, *objects, **named_attrs):
	targets = (named_attrs,) + objects
	msg = {}

	addCCField(msg, 'order-id',    targets)
	addCCField(msg, 'txn-type',    targets)
	addCCField(msg, 'txn-status',    targets)
	addCCField(msg, 'origin',    targets)
	addCCField(msg, 'card-type',    targets)
	addCCField(msg, 'start-time',    targets)
	addCCField(msg, 'end-time',    targets)
	addCCField(msg, 'low-amount',    targets)
	addCCField(msg, 'high-amount',    targets)
	addCCField(msg, 'maxcount',    targets)
	addCCField(msg, 'batch-id',    targets)

	# Remove fields with no values
	for key, val in msg.items():
	    if not string.strip(val):
		del msg[key]

	result =  self._SendServer('query', msg)


	results = []
	for key, val in result.items():
	    if key[0] == 'a' and (47 < ord(key[1]) < 58):
		results.append(CyberCashResponse(decodedict(val)))
		
	return results

class CyberCashResponse:
    """This class converts cybercash responses into objects"""
    
    name = "CyberCash Response"
    def __init__(self, message):
	self.__dict__ = message
    
    def __getattr__(self, attr):
	"Translate requests for attributes with underscores to hyphens"
	return self.__dict__[replace(attr, "_", "-")]
    
    def __str__(self):			# Handy for debugging
	ret = "%s\n\n" % self.name
	for item in self.__dict__.items():
	    ret = ret + "%s: %s\n" % item
	    
	return ret

    def strTime(self):
	return convertCCTime(self.time)
	#return self.time

    
    
def encodedict(dict):
    results = []
    for key, value in dict.items():
	results.append('%s=%s' % (quote_plus(key, ''),  quote_plus(value,'')))
    
    return string.join(results, '&')

def decodedict(str):
    dict = {}
    for item in string.split(str, '&'):
	key, value = string.split(item, '=')
	#string.replace(key, '-', '_')
	dict[unquote_plus(key)] = unquote_plus(value)

    return dict

def addCCField(msg, ccfield, objects, required=0):

    # Cyber Cash fields may have hyphens in them which are
    # not allowed in python attribute names.
    attr = string.replace(ccfield, '-', '_')
    for object in objects:
	if type(object) == DictType:
	    if object.has_key(attr): 
		msg[ccfield] = object[attr]
		return 
	else:
	    if hasattr(object, attr): 
		msg[ccfield] = getattr(object, attr)
		return 
    
    if required:
	raise KeyError, 'Required field "%s" is missing.' % attr

def convertCCTime(cctime):
    # Convert the time to pythontyple

    try:
	t = strptime(cctime, '%Y%m%d%H%M%S.000')
   
	if daylight:
	    offset = altzone
	else:
	    offset = timezone

	    # Convert tuple back to seconds from the epoch
	t = localtime(mktime(t) - offset)
	return strftime('%m/%d/%y (%H:%M:%S)', t) 
    except:
	# there seems to be a bug in strptime where
	# it does not want to parse the string if the minutes are 00
	# So instead of wasting time on tracking down the problem, we'll
	# use this quick work around.
	return "%s" % cctime
    
    


    
