## Copyright  1999 CodeIt Computing, All rights reserved.

## By obtaining, using, and/or copying this software and/or its
## associated documentation, you agree that you have read, understood,
## and will comply with the following terms and conditions:

## Permission to use, copy, modify, and distribute this software and its
## associated documentation for any purpose and without fee is hereby
## granted, provided that the above copyright notice appears in all
## copies, and that both that copyright notice and this permission notice
## appear in supporting documentation, and that the name of CodeIt
## Computing not be used in advertising or publicity pertaining to
## distribution of the software without specific, written prior
## permission.

## CODEIT COMPUTING DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS
## SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND
## FITNESS. IN NO EVENT SHALL CODEIT COMPUTING BE LIABLE FOR ANY SPECIAL,
## INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING
## FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
## NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
## WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

""" The Wampum Generator will raise an error if a message sent to the
server is anything other than success. The results of the message, if
any, will be encapsulated in the exception object so that further
handling can be done if required. All the exceptions inherit from the
base class WampumException wich makes it easy to create general erorr
handeling routines.


WampumException
 |
 +-AVSFailed
 |
 +-SuccessDuplicate
 |
 +-PartialSuccess
 |
 +-FailureHard
 |
 +-FailureQ
 |
 +-FailureSWVersion
 |
 +-FailureBadMoney
"""

__version__='$Revision: 1.2 $'[11:-2]


from string import replace


class WampumException:
    name = "WampumException"
    def __init__(self, message):
	self.__dict__ = message
    
    def __getattr__(self, attr):
	"Translate requests for attributes with underscores to hyphens"
	return self.__dict__[replace(attr, "_", "-")]
    
    def __str__(self):
	ret = "%s\n\n " % self.name
	for item in self.__dict__.items():
	    ret = ret + "%s: %s\n" % item
	    
	return ret

    
	    
class AVSFailed(WampumException): 

    """This exception is only raised during a call to process order
    and the require_avs is true"""

    name = "Address Verification Failed"

class SuccessDuplicate(WampumException): 
    
    """Result of a previously succesful transaction"""

    name = "Duplicated Success"


class PartialSuccess(WampumException): 
    
    """Batched with failed transactions"""

    name = "Partial Success"

class FailureHard(WampumException): 
    
    """Failed Transaction; subsequent retry will not help"""

    name = "Failed Transaction"

class FailureQ(WampumException): 
    
    """Failed Transaction do to communication error, this may be
    retired"""

    name = "Failed Transaction do to communication error"


class FailureSWVersion(WampumException):
    
    """Failed transaction because of old or non-existent software
    (version) being used"""

    name = "Bad Software Version"

class FailureBadMoney(WampumException):
     
    """Failure Bad Money, failed transaction because of credit problem
    with the financial institution"""

    name = "Bad Money"
