## Copyright  1999 CodeIt Computing, All rights reserved.

## By obtaining, using, and/or copying this software and/or its
## associated documentation, you agree that you have read, understood,
## and will comply with the following terms and conditions:

## Permission to use, copy, modify, and distribute this software and its
## associated documentation for any purpose and without fee is hereby
## granted, provided that the above copyright notice appears in all
## copies, and that both that copyright notice and this permission notice
## appear in supporting documentation, and that the name of CodeIt
## Computing not be used in advertising or publicity pertaining to
## distribution of the software without specific, written prior
## permission.

## CODEIT COMPUTING DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS
## SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND
## FITNESS. IN NO EVENT SHALL CODEIT COMPUTING BE LIABLE FOR ANY SPECIAL,
## INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING
## FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
## NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
## WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

__doc__='''The Wampum Generator'''


import string
import OFS.SimpleItem, AccessControl.Role

from Globals import HTMLFile, Persistent, MessageDialog
from Acquisition import Implicit
from CC import Register 
from string import split, join



addWampumForm=HTMLFile('dtml/addWampum', globals())


def addWampum(self, id, title, ccps_host, cybercash_id, 
	      merchant_key, REQUEST=None):
    """Method to add a Wampum Generator to Zope"""

    
    self._setObject(id, Wampum(id, title, ccps_host, cybercash_id, merchant_key))
    
    if REQUEST is not None: 
	# Display the folder that contains the new Wampum Generator
	return self.manage_main(self,REQUEST)


class Wampum(
    OFS.SimpleItem.Item,
    Persistent,
    Implicit,
    Register
    ):
    """The Wampum Generator. vWhatever
    
    It does a little of this a little of that.
    """
    
    meta_type='Wampum Generator'	# Objects Name in the Zope list
    icon='misc_/Wampum/icon'   
    manage_options=(		
    	{'label':'Properties',   'action':'manage_properties'},
    	{'label':'Query',        'action':'manage_queryFrame'},
    	{'label':'Batching',     'action':'manage_batchFrame'},
    	{'label':'Direct Input', 'action':'manage_directInputForm'},
    	{'label':'Security',     'action':'manage_access'},
	{'label':'About',        'action':'manage_aboutWampum'},
    	)
    
    __ac_permissions__=(
        ('View management screens', ('manage',)),
        ('Query Cybercash', ('manage_query',)),
        ('Process Credit Card Order',('',)),
        )



    manage_properties  = HTMLFile('dtml/properties', globals())

    # Needed for backwards compatability with zope 1.10.x
    manage=manage_main = manage_properties

    manage_queryFrame  = HTMLFile('dtml/queryFrame', globals())
    manage_queryForm   = HTMLFile('dtml/queryForm', globals())
    manage_queryResult = HTMLFile('dtml/queryResult', globals())

    manage_batchFrame  = HTMLFile('dtml/batchFrame', globals())
    manage_batchForm   = HTMLFile('dtml/batchForm', globals())
    manage_batchResult = HTMLFile('dtml/batchResult', globals())


    manage_directInputForm  = HTMLFile('dtml/directInput', globals())

    manage_aboutWampum = HTMLFile('dtml/about', globals())
              
    def __init__(self, id, title, ccps_host, cybercash_id, merchant_key):
	""" The Wampum constructor function. """
	self.id           = id
	self.title        = title
    
	# Set CyberCash variables to suitable defaults for testing
	# if they are blank.
	self.ccps_host = ccps_host or "http://cr.cybercash.com/cgi-bin/"
	self.cybercash_id = cybercash_id or "test-mck"
	self.merchant_key =  merchant_key or "gmRJp8rbzzcRdI5g8eLRScFeeQSWew5FsBzq6bxJDSJIiWhq7f"
	self.require_avs = 1
	self.txn_model = 'mauthcapture'


    def manage_edit(self, title, ccps_host, cybercash_id, 
		    merchant_key, txn_model, require_avs, REQUEST=None):
	"""Modify Wampum's Properties"""
	self.title        = title
	self.ccps_host    = ccps_host
	self.cybercash_id = cybercash_id
	self.merchant_key = merchant_key
	self.txn_model    = txn_model
	self.require_avs  = require_avs
	if REQUEST is not None:
	    return self.manage_properties(self, REQUEST)


    def manage_query(self, start_day="", start_hour="",
		     end_day="", end_hour="",  
		     REQUEST=None):
	"""Query CyberCash for transactions"""

	try:
	    if start_day:
		if start_hour == "": start_hour = "00:00:00"
		start_time = parseDay(start_day) + parseHour(start_hour)

		REQUEST.set('start_time', start_time)
	except ValueError:
	    raise ValueError, "Invalid Start Time"
	
	try:
	    if end_day:
		if end_hour == "": end_hour = "23:59:59"
		end_time = parseDay(end_day) +  parseHour(end_hour)
		REQUEST.set('end_time', end_time)
	except ValueError:
	    raise ValueError, "Invalid End Time"

	return self.manage_queryResult(self, QueryResults=self.query(REQUEST))

    def manage_order(self, order_id, REQUEST):
	"""Review/Mark/Unmark Orders"""
	
	results = self.query(order_id=order_id)
	
	# Find the last successful transaction for this item.
	last_txn = results[0]
	for txn in results[1:]:
	    if txn.time > last_txn.time and txn.txn_status == 'success':
		last_txn = txn
	
	nextAction = None
	if last_txn.txn_status == 'success':
	    txn_type = last_txn.txn_type
	    if txn_type == 'auth':
		nextAction = '''<center>
		<form action="manage_nextAction" method="post">
		<input type="hidden" name="action" value="mark">
		<input type="hidden" name="order_id" value="%s">
		<b>Mark for batching:</b> Enter amount 
		<input type="text" name="amount" value="%s"> <input type="submit" value="Mark">
		</form>
		<center>''' % (last_txn.order_id, last_txn.amount)
	    elif txn_type == 'marked':
		nextAction = '''<center>
		<form action="manage_nextAction" method="post">
		<input type="hidden" name="action" value="void">
		<input type="hidden" name="order_id" value="%s">
		<b>Prevent batching:</b>
		<input type="submit" value="Unmark"></form>
		</center>''' % last_txn.order_id
	    elif txn_type == 'settled':
		nextAction = '''<center>
		<form action="manage_nextAction" method="post">
		<input type="hidden" name="action" value="return">
		<input type="hidden" name="order_id" value="%s">
		<b>Mark for Return:</b> Enter amount 
		<input type="text" name="amount" value="%s"> <input type="submit" value="Refund">
		</form>
		<center>''' % (last_txn.order_id, last_txn.amount)
	    
	return self.manage_queryResult(self, QueryResults=results, nextAction=nextAction)

    def manage_nextAction(self, action, order_id, REQUEST):
	"""Preform the next logical action on a order"""

	if action == 'mark':
	    self.postauth(REQUEST)
	    msg = "Order: %s was succesfully marked for batching." % order_id
	elif action == 'void':
	    self.void(REQUEST, txn_type='marked')
	    msg = "Order: %s was succesfully voided." % order_id


	return MessageDialog(title='Action succesful',
			     message='''<input type="hidden" name="order_id" value="%s">
			     %s''' % (order_id, msg),
			     action='manage_order')

    def manage_batchprep(self, REQUEST):
	"""Query CyberCash for txn ready to be batched"""
	r = self.batchprep(REQUEST)

	return self.manage_batchResult(self, QueryResults=r)


    def manage_commitBatch(self, ids, REQUEST):
	"""Commit a batch"""

	# <sigh> transaction must be consecutive.
	# LAME LAME LAME!!!! But I'm lazy
	msg = {}
	indexes = [] 
	index = 0
	for id in ids:
	    index = index + 1
	    indexes.append(index)

	    msg['order_id_%s' % index] = REQUEST.form['order_id_%s' % id]
	    msg['txn_type_%s' % index] = REQUEST.form['txn_type_%s' % id]
	    msg['amount_%s' % index] = REQUEST.form['amount_%s' % id]
	    
	
	response = self.batchcommit(indexes, msg)
	return MessageDialog(title='Action succesful',
			     message= "Batch was succesfully submited.",
			     action='manage_batchResult')


	    
    
    def manage_directInput(self, operation, REQUEST):
	"""Directly submit a request"""
		
	if operation == "capture":
	    self.processorder(REQUEST)
	
	return self.manage_directInputForm(self, REQUEST)


def parseHour(strHour):
    try:
	hour, min, sec = split(strHour, ":")
	hour = int(hour)
	min = int(min)
	sec = int(sec)
	
	if (0 <= hour < 24) and ( 0 <= min < 60 ) and (0 <= sec < 60):
	    return "%02d%02d%02d" % (hour, min, sec)
    except:
	raise ValueError
    else:
	raise ValueError


    
def parseDay(strDay):
    # Note: This will catch obvious errors like months > 12
    # etc... But won't catch things like specifying a day of
    # 31 in a month that has only 30 etc...

    try:
	month, day, year = split(strDay, "/")
	month = int(month)
	day = int(day)
	year = int(year)
	if (0 < month < 13) and (0 < day < 32):
	    return "%04d%02d%02d" % (year, month, day)
    except:
	raise ValueError
    else:
	raise ValueError
