/**********************************************************************
 * Copyright  1999 CodeIt Computing, All rights reserved.
 *
 * By obtaining, using, and/or copying this software and/or its
 * associated documentation, you agree that you have read, understood,
 * and will comply with the following terms and conditions:
 *
 * Permission to use, copy, modify, and distribute this software and
 * its associated documentation for any purpose and without fee is
 * hereby granted, provided that the above copyright notice appears in
 * all copies, and that both that copyright notice and this permission
 * notice appear in supporting documentation, and that the name of
 * CodeIt Computing not be used in advertising or publicity pertaining
 * to distribution of the software without specific, written prior
 * permission.
 *
 * CODEIT COMPUTING DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS
 * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS. IN NO EVENT SHALL CODEIT COMPUTING BE LIABLE FOR ANY
 * SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN
 * AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING
 * OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
 * SOFTWARE.
 *********************************************************************/

#include <time.h>
#include <unistd.h>

#include "CCErrno.h"
#include "mckcrypt.h"
#include "Python.h"

#define TMPBUFSIZE 1024

static PyObject *encrypt(PyObject *self, PyObject *args) {
  char* wrappedMerchantKey = NULL;
  char *message = NULL;
  char *response[1024];


  /* use time to generate a session key.... */
  char *mytstamp = NULL;
  char *cp = NULL;
  char  sessionKey[TMPBUFSIZE];

  time_t     mytime;
  int      mypid;
  
  /* results come back here... */
  /* need to malloc the result buffer based on size of */
  /* input string */

  unsigned char* outputBuf = NULL;
  char* encode64Buf = NULL;
  unsigned int   outputBufLen, outputLen;
  unsigned int   msgLen;
  
  unsigned char mac[20];
  long    errcode;

  PyObject *result = NULL;

  if (!PyArg_ParseTuple(args, "ss", &wrappedMerchantKey, &message))
    return NULL;
  

  /* Generate a session key... */
  mytime = time((time_t *) NULL);
  mypid = getpid();
  mytstamp = ctime(&mytime);

  /* Sigh... ctime generates time with a newline that we'll have to zap! */

  if ((cp = (char *)strchr(mytstamp,'\n')) != NULL) {
    *cp = '\0';
  }

  /* likewise for carriage return, if any... */
  if ((cp = (char *)strchr(mytstamp,'\r')) != NULL) {
    *cp = '\0';
  }


  sprintf(sessionKey,"%s #%ld",mytstamp,mypid);
  
	
  msgLen = strlen(message) + 1;  /* include the null byte */
  outputBufLen = msgLen + 10;
  outputBuf = (unsigned char *) malloc(outputBufLen * sizeof(unsigned char));
 
  errcode = mck_encr(wrappedMerchantKey, sessionKey,
		     msgLen, (unsigned char*) message,
		     outputBufLen, outputBuf, &outputLen,
		     mac);

  if (errcode != 0) {
    PyErr_SetString(PyExc_RuntimeError, CCGetErrorMessage(CC_ECRYPTO));
    return NULL;
  }
  
  
  result = Py_BuildValue("ss#s#", sessionKey, mac, 20, outputBuf, outputLen);
  return result;


}

static PyObject *decrypt(PyObject *self, PyObject *args) {
	unsigned int msgStrLen, b64Size ;
	
	char* wrappedMerchantKey = NULL;

	PyObject *pyMessage = NULL;
	char* message = NULL;

	char* sessionKey = NULL;       /* from message */
	PyObject  *mac = NULL;       /* from message */

		/* results come back here... */
		/* need to malloc the result buffer based on size of */
		/* input string */

	unsigned char* rawMsg = NULL;
	unsigned int msgLen;
        unsigned char* outMsg = NULL;
	unsigned int   outputLen, len;


	
	unsigned char  outMAC[20];
	long    errcode;

	PyObject *result = NULL;

	/*  Note: The message needs to be a python string object because it
	    may contain arbitary null bytes in it 
	    
	    Could this be true for the MAC as well?
	*/

	if (!PyArg_ParseTuple(args, "ssSS", &wrappedMerchantKey, &sessionKey, &mac, &pyMessage))
	  return NULL;

	
	msgLen =  PyString_Size (pyMessage); 
	message = PyString_AsString(pyMessage);


	outMsg = (unsigned char *) malloc(msgLen * sizeof(unsigned char));

	errcode = mck_decr (wrappedMerchantKey, sessionKey,
			    msgLen, (unsigned char*) message,
			    msgLen, outMsg, &outputLen,
			    outMAC);


	
	if (errcode != 0) {
	  
	  PyErr_SetString(PyExc_RuntimeError, CCGetErrorMessage(CC_ECRYPTO));
	  goto BAIL;
	}
	
	/* DO the macs agree? */
	if (memcmp( PyString_AsString(mac), outMAC, 20 ) != 0) {
	  PyErr_SetString (PyExc_RuntimeError, "MACs do not match");
	  goto BAIL;
	}

	//It looks like this string comes back NULL terminated
	//result = Py_BuildValue("s#", outMsg, outputLen);
	result = Py_BuildValue("s", outMsg);
	free(outMsg);

	return result;


BAIL:
	free(outMsg);
	return NULL;
}

static struct PyMethodDef ZCCcrypt_module_methods[] = {
  /* Function Name,   C Language Implementation,  flag(always 1)  */
  {"encrypt",            (PyCFunction)encrypt,   METH_VARARGS|METH_KEYWORDS},
  {"decrypt",            (PyCFunction)decrypt,   METH_VARARGS|METH_KEYWORDS},
  {NULL, NULL}			/* sentinel */
};

void initZCCcrypt(){
  Py_InitModule("ZCCcrypt", ZCCcrypt_module_methods);
}
